import unittest
import doctest
import sys
if sys.version_info > (3,):
    long = int

class DeviceTest(unittest.TestCase):
    def runTest(self):
        try:
            import pytrap
        except ImportError as e:
            self.fail(str(e))

class TrapCtxInitTest(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.TrapCtx()
        try:
            c.init()
            self.fail("argv is mandatory")
        except:
            pass
        try:
            c.init([])
            self.fail("argv must not be empty")
        except:
            pass
        try:
            c.init([[]])
            self.fail("argv[i] must be string")
        except:
            pass

        c.init(["-i", "u:test_init"], 0, 1)
        c.finalize()

#Skip tests that print help
#class TrapCtxHelpTest(unittest.TestCase):
#    def runTest(self):
#        import pytrap
#        c = pytrap.TrapCtx()
#        try:
#            c.init(["-h"], 0, 1)
#            self.fail("Calling method of uninitialized context.")
#        except pytrap.TrapHelp:
#            pass
#        c.finalize()
#
#class TrapCtxHelpifcTest(unittest.TestCase):
#    def runTest(self):
#        import pytrap
#        c = pytrap.TrapCtx()
#        try:
#            c.init(["-h", "1"], 0, 1)
#            self.fail("Calling method of uninitialized context.")
#        except pytrap.TrapHelp:
#            pass
#        c.finalize()

class TrapCtxNotInitTestGetdatafmt(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.TrapCtx()
        try:
            c.getDataFmt()
            self.fail("Calling method of uninitialized context.")
        except:
            pass

class TrapCtxNotInitTestSetrequiredfmt(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.TrapCtx()
        try:
            c.setRequiredFmt(0)
            self.fail("Calling method of uninitialized context.")
        except:
            pass

class TrapCtxNotInitTestGetinifcstate(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.TrapCtx()
        try:
            c.getInIFCState(0)
            self.fail("Calling method of uninitialized context.")
        except:
            pass

class TrapCtxNotInitTestIfcctl(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.TrapCtx()
        try:
            c.ifcctl(0, True, CTL_IMEOUT, 0)
            self.fail("Calling method of uninitialized context.")
        except:
            pass

class TrapCtxNotInitTestSend(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.TrapCtx()
        try:
            c.send()
            self.fail("Calling method of uninitialized context.")
        except:
            pass

class TrapCtxNotInitTestSendFlush(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.TrapCtx()
        try:
            c.sendFlush()
            self.fail("Calling method of uninitialized context.")
        except:
            pass

class TrapCtxNotInitTestRecv(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.TrapCtx()
        try:
            c.recv()
            self.fail("Calling method of uninitialized context.")
        except:
            pass

class TrapCtxGetVersion(unittest.TestCase):
    def runTest(self):
        import pytrap
        c = pytrap.getTrapVersion()
        print(c)

class StoreAndLoadMessage(unittest.TestCase):
    def runTest(self):
        import pytrap
        import os

        urtempl = "ipaddr IP,uint16 PORT"
        c = pytrap.TrapCtx()
        c.init(["-i", "f:/tmp/pytrap_test"], 0, 1)
        c.setDataFmt(0, pytrap.FMT_UNIREC, urtempl)

        t = pytrap.UnirecTemplate("ipaddr IP,uint16 PORT")
        t.createMessage()
        t.IP = pytrap.UnirecIPAddr("192.168.0.1")
        t.PORT = 123
        c.send(t.getData())
        c.sendFlush()
        c.finalize()

        c = pytrap.TrapCtx()
        c.init(["-i", "f:/tmp/pytrap_test"], 1)
        c.setRequiredFmt(0, pytrap.FMT_UNIREC, urtempl)
        data = c.recv()
        t = pytrap.UnirecTemplate(urtempl)
        t.setData(data)
        self.assertEqual(t.IP, pytrap.UnirecIPAddr("192.168.0.1"))
        self.assertEqual(t.PORT, 123)
        c.finalize()

        os.unlink("/tmp/pytrap_test")

class StoreAndLoadStringMessage(unittest.TestCase):
    def runTest(self):
        #"""json.dump returns str object, which was formerly not supported by pytrep send()"""
        import pytrap
        import json
        import os

        urtempl = "test"
        jdata = {"SRC_IP": "192.168.0.1"}
        c = pytrap.TrapCtx()
        c.init(["-i", "f:/tmp/pytrap_test2"], 0, 1)
        c.setDataFmt(0, pytrap.FMT_JSON, urtempl)

        c.send(json.dumps(jdata))
        c.sendFlush()
        c.finalize()

        c = pytrap.TrapCtx()
        c.init(["-i", "f:/tmp/pytrap_test2"], 1)
        c.setRequiredFmt(0, pytrap.FMT_JSON, urtempl)
        data = c.recv()
        c.finalize()
        self.assertEqual(jdata, json.loads(data))

        os.unlink("/tmp/pytrap_test2")

class SendAndReceiveMessageList(unittest.TestCase):
    def runTest(self):
        import pytrap
        import os
        import time

        messages = 10000000

        urtempl = "ipaddr IP,uint16 PORT"

        # Start sender
        c1 = pytrap.TrapCtx()
        c1.init(["-i", "f:/tmp/pytrap_test3"], 0, 1)
        c1.setDataFmt(0, pytrap.FMT_UNIREC, urtempl)
        c1.ifcctl(0, False, pytrap.CTL_TIMEOUT, 500000)
        c1.ifcctl(0, False, pytrap.CTL_AUTOFLUSH, 500000)

        t = pytrap.UnirecTemplate(urtempl)
        t.createMessage()

        t.IP = pytrap.UnirecIPAddr("192.168.0.1")

        for i in range(messages):
            t.PORT=i
            c1.send(t.getData())
        c1.sendFlush()

        # Start Receiver
        c2 = pytrap.TrapCtx()
        c2.init(["-i", "f:/tmp/pytrap_test3"], 1)
        c2.setRequiredFmt(0, pytrap.FMT_UNIREC, urtempl)
        startt = time.process_time()
        data = c2.recvBulk(t, time=15, count=messages)
        elapsed_time = time.process_time() - startt
        print(f"recvBulk() Elapsed time for {messages} messages is: {elapsed_time}")

        self.assertEqual(len(data), messages)
        ports = [i["PORT"] for i in data]
        self.assertEqual(ports, [i & 0xFFFF for i in range(messages)])

        # Start Receiver
        c2 = pytrap.TrapCtx()
        c2.init(["-i", "f:/tmp/pytrap_test3"], 1)
        c2.setRequiredFmt(0, pytrap.FMT_UNIREC, urtempl)
        startt = time.process_time()
        data = list()
        while True:
            d = c2.recv()
            if not d:
                break
            t.setData(d)
            data.append(t.getDict())

        elapsed_time = time.process_time() - startt
        print(f"recv() Elapsed time for {messages} messages is: {elapsed_time}")

        self.assertEqual(len(data), messages)
        ports = [i["PORT"] for i in data]
        self.assertEqual(ports, [i & 0xFFFF for i in range(messages)])

        c1.finalize()
        c2.finalize()

        os.unlink("/tmp/pytrap_test3")

class SendBulkTest(unittest.TestCase):
    def runTest(self):
        import os
        import pytrap

        urtempl = "ipaddr IP,uint16 PORT"

        # Start sender
        c1 = pytrap.TrapCtx()
        c1.init(["-i", "f:/tmp/pytrap_sendbulktest"], 0, 1)
        c1.setDataFmt(0, pytrap.FMT_UNIREC, urtempl)
        t = pytrap.UnirecTemplate(urtempl)
        data = (
            {"IP": "10.0.0.1", "PORT": 1},
            {"IP": "10.0.0.2", "PORT": 2},
            {"IP": "10.0.0.3", "PORT": 3},
            {"IP": "10.0.0.4", "PORT": 4},
            {"IP": "10.0.0.1", "PORT": 5},
            {"IP": "10.0.0.2", "PORT": 6}
        )

        c1.sendBulk(t, data)

        c1.sendFlush()

        c1.finalize()

        os.unlink("/tmp/pytrap_sendbulktest")
        



import pytrap

def read_nemea(ifc_spec, nrows=-1, array=False):
    """
    Read `nrows` records from NEMEA TRAP interface given by `ifc_spec` and convert then into Pandas DataFrame.

    Example:
        >>> import pytrap
        >>> pytrap.read_nemea("f:/tmp/testunirechelper", array=True)
        [{'SRC_IP': UnirecIPAddr('10.0.0.255'), 'SRC_PORT': 0}, {'SRC_IP': UnirecIPAddr('10.0.0.255'), 'SRC_PORT': 1}, ...]

    Args:
        ifc_spec (str): IFC specifier for TRAP input IFC, see https://nemea.liberouter.org/trap-ifcspec/
        nrows (int): Number of records, read until end of stream (zero size message) if -1.
        array (bool): Set output type to list of dictionary instead of pandas.DataFrame

    Returns:
        pandas.DataFrame or list of dictionary: DataFrame if array is False, otherwise, list of dictionary

    Raises:
        ModuleNotFoundError: When pandas is not installed.
    """
    c = pytrap.TrapCtx()
    c.init(["-i", ifc_spec], 1, 0)
    c.setRequiredFmt(0)
    rec = None
    l = list()
    while nrows != 0:
        try:
            data = c.recv()
        except pytrap.FormatChanged as e:
            fmttype, fmtspec = c.getDataFmt(0)
            rec = pytrap.UnirecTemplate(fmtspec)
            data = e.data
        if len(data) <= 1:
            break
        rec.setData(data)
        d = rec.getDict()
        if d:
            l.append(d)
        if nrows > 0:
            nrows = nrows - 1
    c.finalize()
    del(c)
    if array:
        return l
    else:
        import pandas as pd
        return pd.DataFrame(l)

class PytrapHelperStore():
    """
    Easy to use helper for pytrap context with one OUTPUT interface.
    It provides data_start() to prepare everything, store_record() to send
    prepared UniRec record, data_stop() to finalize everything.

    The class contains UnirecTemplate accessible via `rec`

    Example 1:

    >>> import pytrap
    >>> with pytrap.PytrapHelperStore("f:/tmp/testunirechelper:w", "ipaddr SRC_IP,uint16 SRC_PORT") as s:
    ...     for i in range(10):
    ...         s.rec.setFromDict({"SRC_IP": "10.0.0.1", "SRC_PORT": i})
    ...         s.store_record()

    Example 2:

    >>> import pytrap
    >>> s = pytrap.PytrapHelperStore("f:/tmp/testunirechelper:w", "ipaddr SRC_IP,uint16 SRC_PORT")
    >>> rec = s.data_start()
    >>> for i in range(10):
    ...     rec.setFromDict({"SRC_IP": "10.0.0.255", "SRC_PORT": i})
    ...     s.store_record()
    >>> s.data_stop()

    """
    def __init__(self, ifcspec, fmtspec):
        """Create new instance of pytrap context with one output interface to send/store data.

        Args:
            ifcspec (str): TRAP specifier for output interface, see https://nemea.liberouter.org/trap-ifcspec/
            fmtspec (str): UniRec template specifier
        """
        self.ifcspec = ifcspec
        self.fmtspec = fmtspec

    def __enter__(self):
        self.data_start()
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        self.data_stop()

    def data_start(self):
        """Start storing the test records into self.filename - initialization."""
        fmttype = pytrap.FMT_UNIREC
        self.rec = pytrap.UnirecTemplate(self.fmtspec)
        self.rec.createMessage(0)

        self.trap = pytrap.TrapCtx()
        self.trap.init(["-i", self.ifcspec], 0, 1)
        self.trap.setDataFmt(0, fmttype, self.fmtspec)
        return self.rec

    def store_record(self):
        """Store one UniRec record that was filled outside."""
        if self.trap:
            self.trap.send(self.rec.getData())

    def data_stop(self):
        """Finish storing test data."""
        if self.trap:
            self.trap.finalize()

class PytrapHelperLoad():
    """
    Easy to use helper for pytrap context with one INPUT interface.
    It provides data_start() to prepare everything, load_record() to receive
    and return one UniRec record inside UnirecTemplate, data_stop() to finalize
    everything.

    Example 1:

    >>> import pytrap
    >>> with pytrap.PytrapHelperLoad("f:/tmp/testunirechelper") as s:
    ...     rec = s.load_record()
    ...     while rec:
    ...         # Do something with record:
    ...         # print(rec.strRecord())
    ...         #
    ...         # Load the next record:
    ...         rec = s.load_record()

    Example 2:

    >>> import pytrap
    >>> s = pytrap.PytrapHelperLoad("f:/tmp/testunirechelper")
    >>> s.data_start()
    >>> rec = s.load_record()
    >>> while rec:
    ...    # Do something with record:
    ...    # print(rec.strRecord())
    ...    #
    ...    # Load the next record:
    ...    rec = s.load_record()
    >>> s.data_stop()

    """
    def __init__(self, ifcspec):
        self.ifcspec = ifcspec

    def __enter__(self):
        self.data_start()
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        self.data_stop()

    def data_start(self):
        """Start evaluation of the stored data - initialization."""
        self.trap = pytrap.TrapCtx()
        self.trap.init(["-i", self.ifcspec], 1, 0)
        self.trap.setRequiredFmt(0, pytrap.FMT_UNIREC)

    def load_record(self):
        """Load one UniRec record and return UnirecTemplate if the record was
        received. Otherwise, return None when no more data."""
        try:
            data = self.trap.recv()
        except pytrap.FormatChanged as err:
            fmttype, fmtspec = self.trap.getDataFmt(0)
            self.rec = pytrap.UnirecTemplate(fmtspec)
            data = err.data
        if len(data) <= 1:
            # empty message - do not process it!!!
            return None

        self.rec.setData(data)
        return self.rec

    def data_stop(self):
        """Finish evaluation."""
        if self.trap:
            self.trap.finalize()

if __name__ == "__main__":
    import doctest
    doctest.testmod()




from setuptools import setup, Extension
import os

SRC_PATH = os.path.relpath(os.path.join(os.path.dirname(__file__), ".", "src"))

name = 'nemea-pytrap'
version = '0.15.0'
release = version
description = 'Python extension of the NEMEA project.'
long_description = 'The pytrap module is a native Python extension that allows for writing NEMEA modules in Python.'
author = 'Tomas Cejka'
author_email = 'cejkat@cesnet.cz'
maintainer = 'Tomas Cejka'
maintainer_email = 'cejkat@cesnet.cz'

pytrapmodule = Extension('pytrap.pytrap',
                    sources = ['src/pytrapmodule.c', 'src/unirecmodule.c', 'src/unirecipaddr.c', 'src/unirecmacaddr.c', 'src/fields.c'],
                    libraries = ['trap', 'unirec'])

try:
    from sphinx.setup_command import BuildDoc
    cmdclass = {'build_sphinx': BuildDoc}
    command_options = {'build_sphinx': {
               'project': ('setup.py', name),
               'version': ('setup.py', version),
               'release': ('setup.py', release),
               'source_dir': ('setup.py', 'docs'),
               'build_dir': ('setup.py', 'dist/doc/')}
               }
except:
    cmdclass = {}
    command_options = {}

setup(name = name,
       version = version,
       description = description,
       long_description = long_description,
       author = author,
       author_email = author_email,
       maintainer = maintainer,
       maintainer_email = maintainer_email,
       url = 'https://github.com/CESNET/Nemea-Framework',
       license = 'BSD',
       test_suite = "test",
       platforms = ["Linux"],
       classifiers = [
              'Development Status :: 4 - Beta',
              'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
              'Operating System :: POSIX :: Linux',
              'Programming Language :: C',
              'Programming Language :: Python :: 3',
              'Programming Language :: Python :: 3.6',
              'Programming Language :: Python :: Implementation :: CPython',
              'Topic :: Software Development :: Libraries',
              'Topic :: System :: Networking :: Monitoring'
       ],
       ext_modules = [pytrapmodule],
       packages = ["pytrap"],
       package_dir={ "": SRC_PATH, },
       # sphinx:
       cmdclass=cmdclass,
       # these are optional and override conf.py settings
       command_options=command_options,
       )

